# coding: utf-8

"""
    Stadia Maps Geospatial APIs

    The Stadia Maps Geospatial APIs provide you with the data you need to build awesome applications.

    The version of the OpenAPI document: 9.2.1
    Contact: support@stadiamaps.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from stadiamaps.models.locate_detailed_edge import LocateDetailedEdge
from stadiamaps.models.locate_edge_info import LocateEdgeInfo
from stadiamaps.models.node_id import NodeId
from typing import Optional, Set
from typing_extensions import Self

class LocateEdge(BaseModel):
    """
    LocateEdge
    """ # noqa: E501
    edge_id: Optional[NodeId] = None
    way_id: Optional[StrictInt] = Field(default=None, description="The OSM way ID associated with this edge (absent in verbose response; see the edge info).")
    correlated_lat: Union[StrictFloat, StrictInt]
    correlated_lon: Union[StrictFloat, StrictInt]
    percent_along: Union[StrictFloat, StrictInt]
    side_of_street: StrictStr
    linear_reference: Optional[StrictStr] = Field(default=None, description="A base64-encoded [OpenLR location reference](https://www.openlr-association.com/fileadmin/user_upload/openlr-whitepaper_v1.5.pdf), for a graph edge of the road network matched by the query.")
    outbound_reach: Optional[StrictInt] = None
    heading: Optional[Union[StrictFloat, StrictInt]] = None
    inbound_reach: Optional[StrictInt] = None
    distance: Optional[Union[StrictFloat, StrictInt]] = None
    predicted_speeds: Optional[List[StrictInt]] = Field(default=None, description="Predicted speed information based on historical data. If available, this will include 2016 entries. Each entry represents 5 minutes, where the first entry represents midnight on Monday, the second entry represents 00:05 on Monday, etc.")
    edge_info: Optional[LocateEdgeInfo] = None
    edge: Optional[LocateDetailedEdge] = None
    warnings: Optional[List[StrictStr]] = None
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["edge_id", "way_id", "correlated_lat", "correlated_lon", "percent_along", "side_of_street", "linear_reference", "outbound_reach", "heading", "inbound_reach", "distance", "predicted_speeds", "edge_info", "edge", "warnings"]

    @field_validator('side_of_street')
    def side_of_street_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['left', 'right', 'neither']):
            raise ValueError("must be one of enum values ('left', 'right', 'neither')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of LocateEdge from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of edge_id
        if self.edge_id:
            _dict['edge_id'] = self.edge_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of edge_info
        if self.edge_info:
            _dict['edge_info'] = self.edge_info.to_dict()
        # override the default output from pydantic by calling `to_dict()` of edge
        if self.edge:
            _dict['edge'] = self.edge.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of LocateEdge from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "edge_id": NodeId.from_dict(obj["edge_id"]) if obj.get("edge_id") is not None else None,
            "way_id": obj.get("way_id"),
            "correlated_lat": obj.get("correlated_lat"),
            "correlated_lon": obj.get("correlated_lon"),
            "percent_along": obj.get("percent_along"),
            "side_of_street": obj.get("side_of_street"),
            "linear_reference": obj.get("linear_reference"),
            "outbound_reach": obj.get("outbound_reach"),
            "heading": obj.get("heading"),
            "inbound_reach": obj.get("inbound_reach"),
            "distance": obj.get("distance"),
            "predicted_speeds": obj.get("predicted_speeds"),
            "edge_info": LocateEdgeInfo.from_dict(obj["edge_info"]) if obj.get("edge_info") is not None else None,
            "edge": LocateDetailedEdge.from_dict(obj["edge"]) if obj.get("edge") is not None else None,
            "warnings": obj.get("warnings")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


